pub enum PlannerType {
Hover(HoverPlanner),
MinimumJerkLine(MinimumJerkLinePlanner),
Lissajous(LissajousPlanner),
Circle(CirclePlanner),
Landing(LandingPlanner),
ObstacleAvoidance(ObstacleAvoidancePlanner),
MinimumSnapWaypoint(MinimumSnapWaypointPlanner),
QPpolyTraj(QPpolyTrajPlanner),
}Expand description
Enum representing different types of trajectory planners
Example
use peng_quad::PlannerType;
use peng_quad::HoverPlanner;
let hover_planner : PlannerType = PlannerType::Hover(HoverPlanner{
target_position: nalgebra::Vector3::new(0.0, 0.0, 1.0),
target_yaw: 0.0,
});Variants
Hover(HoverPlanner)
Hover planner
MinimumJerkLine(MinimumJerkLinePlanner)
Minimum jerk line planner
Lissajous(LissajousPlanner)
Minimum jerk circle planner
Circle(CirclePlanner)
Minimum jerk circle planner
Landing(LandingPlanner)
Minimum jerk landing planner
ObstacleAvoidance(ObstacleAvoidancePlanner)
Obstacle avoidance planner
MinimumSnapWaypoint(MinimumSnapWaypointPlanner)
Minimum snap waypoint planner
QPpolyTraj(QPpolyTrajPlanner)
Quadratic Polynomial based waypoint planner
Implementations
Sourceimpl PlannerType
Implementation of the planner type
impl PlannerType
Implementation of the planner type
Sourcepub fn plan(
&self,
current_position: Vector3<f32>,
current_velocity: Vector3<f32>,
time: f32,
) -> (Vector3<f32>, Vector3<f32>, f32)
pub fn plan( &self, current_position: Vector3<f32>, current_velocity: Vector3<f32>, time: f32, ) -> (Vector3<f32>, Vector3<f32>, f32)
Plans the trajectory based on the current planner type
Arguments
current_position- The current position of the quadrotorcurrent_velocity- The current velocity of the quadrotortime- The current simulation time
Returns
- A tuple containing the desired position, velocity, and yaw angle
Example
use nalgebra::Vector3;
use peng_quad::PlannerType;
use peng_quad::HoverPlanner;
let hover_planner = HoverPlanner {
target_position: Vector3::new(0.0, 0.0, 1.0),
target_yaw: 0.0
};
let hover_planner_type = PlannerType::Hover(hover_planner);
let (desired_position, desired_velocity, desired_yaw) = hover_planner_type.plan(Vector3::new(0.0, 0.0, 0.0), Vector3::new(0.0, 0.0, 0.0), 0.0);Sourcepub fn is_finished(
&self,
current_position: Vector3<f32>,
time: f32,
) -> Result<bool, SimulationError>
pub fn is_finished( &self, current_position: Vector3<f32>, time: f32, ) -> Result<bool, SimulationError>
Checks if the current trajectory is finished
Arguments
current_position- The current position of the quadrotortime- The current simulation time
Returns
trueif the trajectory is finished,falseotherwise
Example
use nalgebra::Vector3;
use peng_quad::PlannerType;
use peng_quad::HoverPlanner;
use peng_quad::Planner;
let hover_planner = HoverPlanner{
target_position: Vector3::new(0.0, 0.0, 1.0),
target_yaw: 0.0,
};
let is_finished = hover_planner.is_finished(Vector3::new(0.0, 0.0, 0.0), 0.0);Auto Trait Implementations
impl Freeze for PlannerType
impl RefUnwindSafe for PlannerType
impl Send for PlannerType
impl Sync for PlannerType
impl Unpin for PlannerType
impl UnsafeUnpin for PlannerType
impl UnwindSafe for PlannerType