pub struct Imu {
pub accel_bias: Vector3<f32>,
pub gyro_bias: Vector3<f32>,
pub accel_noise_std: f32,
pub gyro_noise_std: f32,
pub accel_bias_std: f32,
pub gyro_bias_std: f32,
/* private fields */
}Expand description
Represents an Inertial Measurement Unit (IMU) with bias and noise characteristics
Example
use nalgebra::Vector3;
use peng_quad::Imu;
let accel_noise_std = 0.0003;
let gyro_noise_std = 0.02;
let accel_bias_std = 0.0001;
let gyro_bias_std = 0.001;
let imu = Imu::new(accel_noise_std, gyro_noise_std, accel_bias_std, gyro_bias_std);Fields
accel_bias: Vector3<f32>Accelerometer bias
gyro_bias: Vector3<f32>Gyroscope bias
accel_noise_std: f32Standard deviation of accelerometer noise
gyro_noise_std: f32Standard deviation of gyroscope noise
accel_bias_std: f32Standard deviation of accelerometer bias drift
gyro_bias_std: f32Standard deviation of gyroscope bias drift
Implementations
Sourceimpl Imu
Implements the IMU
impl Imu
Implements the IMU
Sourcepub fn new(
accel_noise_std: f32,
gyro_noise_std: f32,
accel_bias_std: f32,
gyro_bias_std: f32,
) -> Result<Self, SimulationError>
pub fn new( accel_noise_std: f32, gyro_noise_std: f32, accel_bias_std: f32, gyro_bias_std: f32, ) -> Result<Self, SimulationError>
Creates a new IMU with default parameters
Arguments
accel_noise_std- Standard deviation of accelerometer noisegyro_noise_std- Standard deviation of gyroscope noiseaccel_bias_std- Standard deviation of accelerometer bias driftgyro_bias_std- Standard deviation of gyroscope bias drift
Returns
- A new Imu instance
Example
use peng_quad::Imu;
let imu = Imu::new(0.01, 0.01, 0.01, 0.01);Sourcepub fn read(
&mut self,
true_acceleration: Vector3<f32>,
true_angular_velocity: Vector3<f32>,
) -> Result<(Vector3<f32>, Vector3<f32>), SimulationError>
pub fn read( &mut self, true_acceleration: Vector3<f32>, true_angular_velocity: Vector3<f32>, ) -> Result<(Vector3<f32>, Vector3<f32>), SimulationError>
Simulates IMU readings with added bias and noise
The added bias and noise are based on normal distributions
Arguments
true_acceleration- The true acceleration vectortrue_angular_velocity- The true angular velocity vector
Returns
- A tuple containing the measured acceleration and angular velocity
Errors
- Returns a SimulationError if the IMU readings cannot be calculated
Example
use nalgebra::Vector3;
use peng_quad::Imu;
let mut imu = Imu::new(0.01, 0.01, 0.01, 0.01).unwrap();
let true_acceleration = Vector3::new(0.0, 0.0, 9.81);
let true_angular_velocity = Vector3::new(0.0, 0.0, 0.0);
let (measured_acceleration, measured_ang_velocity) = imu.read(true_acceleration, true_angular_velocity).unwrap();Auto Trait Implementations
impl Freeze for Imu
impl RefUnwindSafe for Imu
impl Send for Imu
impl Sync for Imu
impl Unpin for Imu
impl UnsafeUnpin for Imu
impl UnwindSafe for Imu