pub struct Maze {
pub lower_bounds: [f32; 3],
pub upper_bounds: [f32; 3],
pub obstacles: Vec<Obstacle>,
pub obstacles_velocity_bounds: [f32; 3],
pub obstacles_radius_bounds: [f32; 2],
pub rng: ChaCha8Rng,
}Expand description
Represents a maze in the simulation
Example
use peng_quad::{Maze, Obstacle};
use rand_chacha::ChaCha8Rng;
use rand::SeedableRng;
use nalgebra::Vector3;
let maze = Maze {
lower_bounds: [0.0, 0.0, 0.0],
upper_bounds: [1.0, 1.0, 1.0],
obstacles: vec![Obstacle::new(Vector3::new(0.0, 0.0, 0.0), Vector3::new(0.0, 0.0, 0.0), 1.0)],
obstacles_velocity_bounds: [0.0, 0.0, 0.0],
obstacles_radius_bounds: [0.0, 0.0],
rng: ChaCha8Rng::from_rng(&mut rand::rng()),
};Fields
lower_bounds: [f32; 3]The lower bounds of the maze in the x, y, and z directions
upper_bounds: [f32; 3]The upper bounds of the maze in the x, y, and z directions
obstacles: Vec<Obstacle>The obstacles in the maze
obstacles_velocity_bounds: [f32; 3]The bounds of the obstacles’ velocity
obstacles_radius_bounds: [f32; 2]The bounds of the obstacles’ radius
rng: ChaCha8RngRng for generating random numbers
Implementations
Sourceimpl Maze
Implementation of the maze
impl Maze
Implementation of the maze
Sourcepub fn new(
lower_bounds: [f32; 3],
upper_bounds: [f32; 3],
num_obstacles: usize,
obstacles_velocity_bounds: [f32; 3],
obstacles_radius_bounds: [f32; 2],
) -> Self
pub fn new( lower_bounds: [f32; 3], upper_bounds: [f32; 3], num_obstacles: usize, obstacles_velocity_bounds: [f32; 3], obstacles_radius_bounds: [f32; 2], ) -> Self
Creates a new maze with the given bounds and number of obstacles
Arguments
lower_bounds- The lower bounds of the mazeupper_bounds- The upper bounds of the mazenum_obstacles- The number of obstacles in the maze
Returns
- The new maze instance
Example
use peng_quad::Maze;
let maze = Maze::new([-1.0, -1.0, -1.0], [1.0, 1.0, 1.0], 5, [0.1, 0.1, 0.1], [0.1, 0.5]);Sourcepub fn generate_obstacles(&mut self, num_obstacles: usize)
pub fn generate_obstacles(&mut self, num_obstacles: usize)
Sourcepub fn update_obstacles(&mut self, dt: f32)
pub fn update_obstacles(&mut self, dt: f32)