pub struct PlannerManager {
pub current_planner: PlannerType,
}Expand description
Manages different trajectory planners and switches between them
Example
use nalgebra::Vector3;
use peng_quad::PlannerManager;
let initial_position = Vector3::new(0.0, 0.0, 1.0);
let initial_yaw = 0.0;
let planner_manager = PlannerManager::new(initial_position, initial_yaw);Fields
current_planner: PlannerTypeThe current planner
Implementations
Sourceimpl PlannerManager
Implementation of the PlannerManager
impl PlannerManager
Implementation of the PlannerManager
Sourcepub fn new(initial_position: Vector3<f32>, initial_yaw: f32) -> Self
pub fn new(initial_position: Vector3<f32>, initial_yaw: f32) -> Self
Creates a new PlannerManager with an initial hover planner
Arguments
initial_position- The initial position for hoveringinitial_yaw- The initial yaw angle for hovering
Returns
- A new PlannerManager instance
Example
use nalgebra::Vector3;
use peng_quad::PlannerManager;
let initial_position = Vector3::new(0.0, 0.0, 1.0);
let initial_yaw = 0.0;
let planner_manager = PlannerManager::new(initial_position, initial_yaw);Sourcepub fn set_planner(&mut self, new_planner: PlannerType)
pub fn set_planner(&mut self, new_planner: PlannerType)
Sets a new planner
Arguments
new_planner- The new planner to be set
Example
use nalgebra::Vector3;
use peng_quad::{PlannerManager, CirclePlanner, PlannerType};
let initial_position = Vector3::new(0.0, 0.0, 1.0);
let initial_yaw = 0.0;
let mut planner_manager = PlannerManager::new(initial_position, initial_yaw);
let new_planner = CirclePlanner {
center: Vector3::new(0.0, 0.0, 1.0),
radius: 1.0,
angular_velocity: 1.0,
start_yaw: 0.0,
end_yaw: 0.0,
start_time: 0.0,
duration: 10.0,
ramp_time: 1.0,
start_position: Vector3::new(0.0, 0.0, 1.0),
};
planner_manager.set_planner(PlannerType::Circle(new_planner));Sourcepub fn update(
&mut self,
current_position: Vector3<f32>,
current_orientation: UnitQuaternion<f32>,
current_velocity: Vector3<f32>,
time: f32,
obstacles: &[Obstacle],
) -> Result<(Vector3<f32>, Vector3<f32>, f32), SimulationError>
pub fn update( &mut self, current_position: Vector3<f32>, current_orientation: UnitQuaternion<f32>, current_velocity: Vector3<f32>, time: f32, obstacles: &[Obstacle], ) -> Result<(Vector3<f32>, Vector3<f32>, f32), SimulationError>
Updates the current planner and returns the desired position, velocity, and yaw
Arguments
current_position- The current position of the quadrotorcurrent_orientation- The current orientation of the quadrotorcurrent_velocity- The current velocity of the quadrotortime- The current simulation time
Returns
- A tuple containing the desired position, velocity, and yaw angle
Errors
- Returns a SimulationError if the current planner is not finished
Example
use nalgebra::{Vector3, UnitQuaternion};
use peng_quad::{PlannerManager, SimulationError};
let initial_position = Vector3::new(0.0, 0.0, 1.0);
let initial_yaw = 0.0;
let mut planner_manager = PlannerManager::new(initial_position, initial_yaw);
let current_position = Vector3::new(0.0, 0.0, 1.0);
let current_orientation = UnitQuaternion::from_euler_angles(0.0, 0.0, 0.0);
let current_velocity = Vector3::new(0.0, 0.0, 0.0);
let obstacles = vec![];
let time = 0.0;
let result = planner_manager.update(current_position, current_orientation, current_velocity, time, &obstacles);
match result {
Ok((target_position, target_velocity, target_yaw)) => {
println!("Target Position: {:?}", target_position);
println!("Target Velocity: {:?}", target_velocity);
println!("Target Yaw: {:?}", target_yaw);
}
Err(SimulationError) => {
log::error!("Error: Planner is not finished");
}
}Auto Trait Implementations
impl Freeze for PlannerManager
impl RefUnwindSafe for PlannerManager
impl Send for PlannerManager
impl Sync for PlannerManager
impl Unpin for PlannerManager
impl UnsafeUnpin for PlannerManager
impl UnwindSafe for PlannerManager